@@ -2,15 +2,21 @@
 // for details. All rights reserved. Use of this source code is governed by a
 // BSD-style license that can be found in the LICENSE file.
 
+// @dart = 2.6
+
 part of dart.io;
 
 /**
  * [InternetAddressType] is the type an [InternetAddress]. Currently,
- * IP version 4 (IPv4) and IP version 6 (IPv6) are supported.
+ * IP version 4 (IPv4), IP version 6 (IPv6) and Unix domain address are
+ * supported. Unix domain sockets are available only on Linux, MacOS and
+ * Android.
  */
 class InternetAddressType {
   static const InternetAddressType IPv4 = const InternetAddressType._(0);
   static const InternetAddressType IPv6 = const InternetAddressType._(1);
+  @Since("2.8")
+  static const InternetAddressType unix = const InternetAddressType._(2);
   static const InternetAddressType any = const InternetAddressType._(-1);
 
   @Deprecated("Use IPv4 instead")
@@ -25,8 +31,9 @@ class InternetAddressType {
   const InternetAddressType._(this._value);
 
   factory InternetAddressType._from(int value) {
-    if (value == 0) return IPv4;
-    if (value == 1) return IPv6;
+    if (value == IPv4._value) return IPv4;
+    if (value == IPv6._value) return IPv6;
+    if (value == unix._value) return unix;
     throw new ArgumentError("Invalid type: $value");
   }
 
@@ -41,6 +48,8 @@ class InternetAddressType {
         return "IPv4";
       case 1:
         return "IPv6";
+      case 2:
+        return "Unix";
       default:
         throw new ArgumentError("Invalid InternetAddress");
     }
@@ -50,7 +59,7 @@ class InternetAddressType {
 }
 
 /**
- * An internet address.
+ * An internet address or a Unix domain address.
  *
  * This object holds an internet address. If this internet address
  * is the result of a DNS lookup, the address also holds the hostname
@@ -93,29 +102,37 @@ abstract class InternetAddress {
   external static InternetAddress get ANY_IP_V6;
 
   /**
-   * The [type] of the [InternetAddress] specified what IP protocol.
+   * The address family of the [InternetAddress].
    */
   InternetAddressType get type;
 
   /**
-   * The numeric address of the host. For IPv4 addresses this is using
-   * the dotted-decimal notation. For IPv6 it is using the
-   * hexadecimal representation.
+   * The numeric address of the host.
+   *
+   * For IPv4 addresses this is using the dotted-decimal notation.
+   * For IPv6 it is using the hexadecimal representation.
+   * For Unix domain addresses, this is a file path.
    */
   String get address;
 
   /**
-   * The host used to lookup the address. If there is no host
-   * associated with the address this returns the numeric address.
+   * The host used to lookup the address.
+   *
+   * If there is no host associated with the address this returns the [address].
    */
   String get host;
 
   /**
-   * Get the raw address of this [InternetAddress]. The result is either a
-   * 4 or 16 byte long list. The returned list is a copy, making it possible
-   * to change the list without modifying the [InternetAddress].
+   * The raw address of this [InternetAddress].
+   *
+   * For an IP address, the result is either a 4 or 16 byte long list.
+   * For a Unix domain address, UTF-8 encoded byte sequences that represents
+   * [address] is returned.
+   *
+   * The returned list is a fresh copy, making it possible to change the list without
+   * modifying the [InternetAddress].
    */
-  List<int> get rawAddress;
+  Uint8List get rawAddress;
 
   /**
    * Returns true if the [InternetAddress] is a loopback address.
@@ -133,23 +150,54 @@ abstract class InternetAddress {
   bool get isMulticast;
 
   /**
-   * Creates a new [InternetAddress] from a numeric address.
+   * Creates a new [InternetAddress] from a numeric address or a file path.
    *
-   * If the address in [address] is not a numeric IPv4
-   * (dotted-decimal notation) or IPv6 (hexadecimal representation).
-   * address [ArgumentError] is thrown.
+   * If [type] is [InternetAddressType.IPv4], [address] must be a numeric IPv4
+   * address (dotted-decimal notation).
+   * If [type] is [InternetAddressType.IPv6], [address] must be a numeric IPv6
+   * address (hexadecimal notation).
+   * If [type] is [InternetAddressType.unix], [address] must be a a valid file
+   * path.
+   * If [type] is omitted, [address] must be either a numeric IPv4 or IPv6
+   * address and the type is inferred from the format.
+   *
+   * To create a Unix domain address, [type] should be
+   * [InternetAddressType.unix] and [address] should be a string.
    */
-  external factory InternetAddress(String address);
+  external factory InternetAddress(String address,
+      {@Since("2.8") InternetAddressType type});
 
   /**
-   * Perform a reverse dns lookup on the [address], creating a new
-   * [InternetAddress] where the host field set to the result.
+   * Creates a new [InternetAddress] from the provided raw address bytes.
+   *
+   * If the [type] is [InternetAddressType.IPv4], the [rawAddress] must have
+   * length 4.
+   * If the [type] is [InternetAddressType.IPv6], the [rawAddress] must have
+   * length 16.
+   * If the [type] is [InternetAddressType.IPv4], the [rawAddress] must be a
+   * valid UTF-8 encoded file path.
+   *
+   * If [type] is omitted, the [rawAddress] must have a length of either 4 or
+   * 16, in which case the type defaults to [InternetAddress.IPv4] or
+   * [InternetAddress.IPv6] respectively.
+   */
+  external factory InternetAddress.fromRawAddress(Uint8List rawAddress,
+      {@Since("2.8") InternetAddressType type});
+
+  /**
+   * Perform a reverse DNS lookup on this [address]
+   *
+   * Returns a new [InternetAddress] with the same address, but where the [host]
+   * field set to the result of the lookup.
+   *
+   * If this address is Unix domain addresses, no lookup is performed and this
+   * address is returned directly.
    */
   Future<InternetAddress> reverse();
 
   /**
    * Lookup a host, returning a Future of a list of
-   * [InternetAddress]s. If [type] is [InternetAddressType.ANY], it
+   * [InternetAddress]s. If [type] is [InternetAddressType.any], it
    * will lookup both IP version 4 (IPv4) and IP version 6 (IPv6)
    * addresses. If [type] is either [InternetAddressType.IPv4] or
    * [InternetAddressType.IPv6] it will only lookup addresses of the
@@ -327,7 +375,18 @@ abstract class ServerSocket implements Stream<Socket> {
    * distributed among all the bound `ServerSocket`s. Connections can be
    * distributed over multiple isolates this way.
    */
-  external static Future<ServerSocket> bind(address, int port,
+  static Future<ServerSocket> bind(address, int port,
+      {int backlog: 0, bool v6Only: false, bool shared: false}) {
+    final IOOverrides overrides = IOOverrides.current;
+    if (overrides == null) {
+      return ServerSocket._bind(address, port,
+          backlog: backlog, v6Only: v6Only, shared: shared);
+    }
+    return overrides.serverSocketBind(address, port,
+        backlog: backlog, v6Only: v6Only, shared: shared);
+  }
+
+  external static Future<ServerSocket> _bind(address, int port,
       {int backlog: 0, bool v6Only: false, bool shared: false});
 
   /**
@@ -406,15 +465,16 @@ enum _RawSocketOptions {
   IPPROTO_UDP, // 6
 }
 
-/// The [RawSocketOption] is used as a parameter to [Socket.setRawOption] and
-/// [RawSocket.setRawOption] to set customize the behaviour of the underlying
-/// socket.
+/// The [RawSocketOption] is used as a parameter to [Socket.setRawOption],
+/// [RawSocket.setRawOption], and [RawDatagramSocket.setRawOption] to customize
+/// the behaviour of the underlying socket.
 ///
-/// It allows for fine grained control of the socket options, and its values will
-/// be passed to the underlying platform's implementation of setsockopt and
-/// getsockopt.
+/// It allows for fine grained control of the socket options, and its values
+/// will be passed to the underlying platform's implementation of `setsockopt`
+/// and `getsockopt`.
+@Since("2.2")
 class RawSocketOption {
-  /// Creates a RawSocketOption for getRawOption andSetRawOption.
+  /// Creates a [RawSocketOption] for `getRawOption` and `setRawOption`.
   ///
   /// All arguments are required and must not be null.
   ///
@@ -424,10 +484,10 @@ class RawSocketOption {
   /// The value argument and its length correspond to the optval and length
   /// arguments on the native call.
   ///
-  /// For a [getRawOption] call, the value parameter will be updated after a
+  /// For a `getRawOption` call, the value parameter will be updated after a
   /// successful call (although its length will not be changed).
   ///
-  /// For a [setRawOption] call, the value parameter will be used set the
+  /// For a `setRawOption` call, the value parameter will be used set the
   /// option.
   const RawSocketOption(this.level, this.option, this.value);
 
@@ -437,8 +497,8 @@ class RawSocketOption {
       value = 0;
     }
     final Uint8List list = Uint8List(4);
-    final buffer = ByteData.view(list.buffer);
-    buffer.setInt32(0, value);
+    final buffer = ByteData.view(list.buffer, list.offsetInBytes);
+    buffer.setInt32(0, value, Endian.host);
     return RawSocketOption(level, option, list);
   }
 
@@ -556,8 +616,13 @@ class ConnectionTask<S> {
 }
 
 /**
- * The [RawSocket] is a low-level interface to a socket, exposing the raw
- * events signaled by the system. It's a [Stream] of [RawSocketEvent]s.
+ * A [RawSocket] is an unbuffered interface to a TCP socket.
+ *
+ * The raw socket delivers the data stream in the same chunks as the underlying
+ * operating system.
+ *
+ * It is not the same as a
+ * [POSIX raw socket](http://man7.org/linux/man-pages/man7/raw.7.html).
  */
 abstract class RawSocket implements Stream<RawSocketEvent> {
   /**
@@ -618,7 +683,7 @@ abstract class RawSocket implements Stream<RawSocketEvent> {
    * available for immediate reading. If no data is available [:null:]
    * is returned.
    */
-  List<int> read([int len]);
+  Uint8List read([int len]);
 
   /**
    * Writes up to [count] bytes of the buffer from [offset] buffer offset to
@@ -633,21 +698,29 @@ abstract class RawSocket implements Stream<RawSocketEvent> {
 
   /**
    * Returns the port used by this socket.
+   *
+   * Throws a [SocketException] if the socket is closed.
    */
   int get port;
 
   /**
    * Returns the remote port connected to by this socket.
+   *
+   * Throws a [SocketException] if the socket is closed.
    */
   int get remotePort;
 
   /**
    * Returns the [InternetAddress] used to connect this socket.
+   *
+   * Throws a [SocketException] if the socket is closed.
    */
   InternetAddress get address;
 
   /**
    * Returns the remote [InternetAddress] connected to by this socket.
+   *
+   * Throws a [SocketException] if the socket is closed.
    */
   InternetAddress get remoteAddress;
 
@@ -685,6 +758,7 @@ abstract class RawSocket implements Stream<RawSocketEvent> {
    *
    * Throws an [OSError] on failure.
    */
+  @Since("2.2")
   Uint8List getRawOption(RawSocketOption option);
 
   /**
@@ -693,6 +767,7 @@ abstract class RawSocket implements Stream<RawSocketEvent> {
    *
    * Throws an [OSError] on failure.
    */
+  @Since("2.2")
   void setRawOption(RawSocketOption option);
 }
 
@@ -702,7 +777,7 @@ abstract class RawSocket implements Stream<RawSocketEvent> {
  * The [Socket] exposes both a [Stream] and a [IOSink] interface, making it
  * ideal for using together with other [Stream]s.
  */
-abstract class Socket implements Stream<List<int>>, IOSink {
+abstract class Socket implements Stream<Uint8List>, IOSink {
   /**
    * Creates a new socket connection to the host and port and returns a [Future]
    * that will complete with either a [Socket] once connected or an error
@@ -770,6 +845,9 @@ abstract class Socket implements Stream<List<int>>, IOSink {
    * available options.
    *
    * Returns [:true:] if the option was set successfully, false otherwise.
+   *
+   * Throws a [SocketException] if the socket has been destroyed or upgraded to
+   * a secure socket.
    */
   bool setOption(SocketOption option, bool enabled);
 
@@ -779,7 +857,8 @@ abstract class Socket implements Stream<List<int>>, IOSink {
    *
    * Returns the [RawSocketOption.value] on success.
    *
-   * Throws an [OSError] on failure.
+   * Throws an [OSError] on failure and a [SocketException] if the socket has
+   * been destroyed or upgraded to a secure socket.
    */
   Uint8List getRawOption(RawSocketOption option);
 
@@ -787,27 +866,38 @@ abstract class Socket implements Stream<List<int>>, IOSink {
    * Use [setRawOption] to customize the [RawSocket]. See [RawSocketOption] for
    * available options.
    *
-   * Throws an [OSError] on failure.
+   * Throws an [OSError] on failure and a [SocketException] if the socket has
+   * been destroyed or upgraded to a secure socket.
    */
   void setRawOption(RawSocketOption option);
 
   /**
-   * Returns the port used by this socket.
+   * The port used by this socket.
+   *
+   * Throws a [SocketException] if the socket is closed.
+   * The port is 0 if the socket is a Unix domain socket.
    */
   int get port;
 
   /**
-   * Returns the remote port connected to by this socket.
+   * The remote port connected to by this socket.
+   *
+   * Throws a [SocketException] if the socket is closed.
+   * The port is 0 if the socket is a Unix domain socket.
    */
   int get remotePort;
 
   /**
-   * Returns the [InternetAddress] used to connect this socket.
+   * The [InternetAddress] used to connect this socket.
+   *
+   * Throws a [SocketException] if the socket is closed.
    */
   InternetAddress get address;
 
   /**
-   * Returns the remote [InternetAddress] connected to by this socket.
+   * The remote [InternetAddress] connected to by this socket.
+   *
+   * Throws a [SocketException] if the socket is closed.
    */
   InternetAddress get remoteAddress;
 
@@ -817,12 +907,12 @@ abstract class Socket implements Stream<List<int>>, IOSink {
 }
 
 /**
- * Datagram package. Data send to and received from datagram sockets
+ * Datagram package. Data sent to and received from datagram sockets
  * contains the internet address and port of the destination or source
  * togeter with the data.
  */
 class Datagram {
-  List<int> data;
+  Uint8List data;
   InternetAddress address;
   int port;
 
@@ -830,12 +920,16 @@ class Datagram {
 }
 
 /**
- * The [RawDatagramSocket] is a low-level interface to an UDP socket,
- * exposing the raw events signaled by the system. It's a [Stream] of
- * [RawSocketEvent]s.
+ * A [RawDatagramSocket] is an unbuffered interface to a UDP socket.
+ *
+ * The raw datagram socket delivers the datagrams in the same chunks as the
+ * underlying operating system. It's a [Stream] of [RawSocketEvent]s.
  *
  * Note that the event [RawSocketEvent.readClosed] will never be
  * received as an UDP socket cannot be closed by a remote peer.
+ *
+ * It is not the same as a
+ * [POSIX raw socket](http://man7.org/linux/man-pages/man7/raw.7.html).
  */
 abstract class RawDatagramSocket extends Stream<RawSocketEvent> {
   /**
